#ifndef IND_STRATEGY_H
#define IND_STRATEGY_H

#include <eo> 
#include <ga.h>

#include "strategy.h" 


#include "strategies/kadlr3am_strategy.h"
#include "strategies/klara_strategy.h"
#include "strategies/knight.h"
#include "strategies/kucci.h"
#include "strategies/my_strategy.h"
#include "strategies/o_strategy.h"
#include "strategies/ondrova_strategy.h"
#include "strategies/prokop_strategy.h"
#include "strategies/strategyPP.h"
#include "strategies/tomik_strategy.h"
#include "strategies/PavelN.h"

/*

Individual encapsulating strategy. 

*/ 

using namespace std;



class StrategyIndividual : public EO<double> {
public:
  StrategyIndividual() { myStrategy = NULL; }
  ~StrategyIndividual() { if (myStrategy) delete myStrategy; } 

  StrategyIndividual(const StrategyIndividual& right) {
    myStrategy = NULL;
    if (right.myStrategy)
      set(right.myStrategy->getName());
  }

  StrategyIndividual& operator=(const StrategyIndividual &right) {
    if (right.myStrategy)
      set(right.myStrategy->getName());
    else 
      myStrategy = NULL;
    return *this;
  } 

  Strategy& strategy() { return *myStrategy; } 

  void set(string name) {
    
    if (myStrategy) delete myStrategy; 
    
    if (name == "RandomStrategy") {  myStrategy = new RandomStrategy(); return; } 
    if (name == "AlwaysCooperate") {  myStrategy = new AlwaysCooperate(); return; } 
    if (name == "AlwaysDeceive") {  myStrategy = new AlwaysDeceive(); return; }
    if (name == "TFT_(CopyPartner)") {  myStrategy = new CopyPartner(); return; }
    if (name == "Alternate") {  myStrategy = new Alternate(); return; }
    if (name == "Pavlov") {  myStrategy = new Pavlov(); return; }
    if (name == "Spite") {  myStrategy = new Spite(); return; }
    if (name == "CCD") {  myStrategy = new CCD(); return; }
    if (name == "DDC") {  myStrategy = new DDC(); return; }
    if (name == "FQuick") {  myStrategy = new FQuick(); return; }
    if (name == "TF2T") {  myStrategy = new TF2T(); return; }
    
    if (name == "Pavel_Nohejl") {  myStrategy = new PavelN(); return; }
    if (name == "kadlr3am_Strategie") {  myStrategy = new KadlecStrategy(); return; }
    if (name == "KlarinaStrategie") {  myStrategy = new KStrategy(); return; }
    if (name == "Knight") {  myStrategy = new Knight(); return; }
    if (name == "Kucci") {  myStrategy = new Kucci(); return; }
    
    if (name == "OndrovaStrategie") {  myStrategy = new OStrategy(); return; }
    if (name == "Ondrova_Super_Strategie") {  myStrategy = new OndrovaStrategie(); return; }
    if (name == "ProkopStrategy") {  myStrategy = new ProkopStrategy(); return; }
    if (name == "Strategie_PP") {  myStrategy = new StrategyPP(); return; }
    if (name == "Tomikova_strategie") {  myStrategy = new Tomik(); return; }
    
    throw std::runtime_error("unknown strategy: "+name);
    
  }  

 

  virtual void printOn(std::ostream &os) const {
    os << myStrategy->getName(); 
  }


  Strategy* myStrategy; // pointer to strategy
};  


typedef StrategyIndividual Indi; 


#endif
